/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages/product/inc/customizegeneratepreview
 *
 * @revision		$Revision: 1.4 $
 */

define("de_epages/product/inc/customizegeneratepreview", [
	"jquery",
	"ep",
	"util/mime",

	"ep/canvas",
	"ep/ajax"
], function($, ep, mime){

	return function(lineItemID, options){
		var o = $.extend({
				maxWidth: 400,
				maxHeight: 400,
				prevWidth: 100,
				prevHeight: 100,
				zIndex: 1001
			}, options),
			previewMimeTypes = 'image/jpeg,image/png,image/gif',
			productImage,
			base64Image,
			base64PreviewImage,
			preloadObject = {},
			previewCanvas = $('<canvas />')
				.attr({
					width: o.maxWidth,
					height: o.maxHeight
				})
				.css({
					display: 'none'
				})
				.appendTo('body'),

			// initialize preview image
			init = function(){
				// load product image and center it in the image container
				productImage = $('<img />')
					.appendTo('body')
					.on('load', function(){
						// calc image dimensions so that it fits into the previewCanvas
						var dimension = recalcDimensions(this, {maxWidth: o.maxWidth, maxHeight: o.maxHeight});
						// render product image to previewCanvas
						previewCanvas.canvas().drawImage(productImage[0], (o.maxWidth-dimension.w)/2, (o.maxHeight-dimension.h)/2, dimension.w, dimension.h);
						// call routine to preload images
						preloadImages();
						// remove image again
						productImage.remove();
					})
					.attr('src', o.image)
					.css({
						position: 'absolute',
						left: '-9999',
						top: '-9999',
						display: 'none'
					});
			},

			// preloads images an call function to generate preview when all images are loaded
			preloadImages = function(){
				for(var key in o.images){
					// if preview is allowed for file -> set entry in preload object
					if(mime.mime(o.images[key].FileName.substr(o.images[key].FileName.lastIndexOf(".")+1), previewMimeTypes)){
						preloadObject[key] = 1;
					}else{
						// no preview allowed -> remove from images array
						delete o.images[key];
					}
				}

				// object for all items which should be rendered
				o.items = {};

				// if no images have to be rendered
				if($.isEmptyObject(o.images)){
					$.extend(o.items, o.texts);
					generatePreview();
				}else{
					$.extend(o.items, o.images, o.texts);

					// iterate over the images array
					for(var key in o.images){
						// generate image object
						o.items[key].image = $('<img />')
							.attr('src', o.images[key].webPath+"/"+o.images[key].FileName)
							.data({
								key: key
							})
							.on('load', function(){
								// image is loaded -> delete entry from preload object
								delete preloadObject[$(this).data('key')];

								// if all images are preloaded
								if($.isEmptyObject(preloadObject)){
									generatePreview();
								}
							});
					}
				}
			},

			generatePreview = function(){
				var aliasByZIndex = [];

				// sort aliases by zIndex (necessary to render items in correct order)
				for(var inputAlias in o.items){
					if(aliasByZIndex[o.items[inputAlias].zIndex-o.zIndex]===undefined){
						aliasByZIndex[o.items[inputAlias].zIndex-o.zIndex] = [];
					}
					aliasByZIndex[o.items[inputAlias].zIndex-o.zIndex].push(inputAlias);
				}

				for(var i=0, iLength=aliasByZIndex.length; i<iLength; i++){					// loop over zIndices
					for(var j=0, jLength=aliasByZIndex[i].length; j<jLength; j++){		// loop over aliases
						var area = o.areas[o.items[aliasByZIndex[i][j]].area],
								item = o.items[aliasByZIndex[i][j]];

						// render image to canvas
						if(item.type === 'file'){
							previewCanvas
								.canvas()
									.save()
									.translate(area.left+item.left+item.width/2, area.top+item.top+item.height/2)
									.rotate(item.angle * Math.PI/180)
									.translate(-item.innerWidth/2, -item.innerHeight/2)
									.drawImage(item.image[0], 0, 0, item.innerWidth, item.innerHeight)
									.restore();
						// render text to canvas
						}else if(item.type === 'text'){
							previewCanvas
								.canvas()
									.save()
									.fillStyle(item.fontColor)
									.translate(area.left+item.left+item.width/2, area.top+item.top+item.height/2)
									.rotate(item.angle * Math.PI/180)
									.translate(-item.innerWidth/2, -item.innerHeight/2)
									.multiline({
										x: 0,
										y: 0,
										phi: 0,
										text: item.text,
										fontSize: item.fontSize,
										fontFamily: item.fontFamily,
										fontUnit: item.fontUnit,
										align: 'left',
										delimiter: "\n"
									})
									.restore();
						}
					}
				}

				// extract base64 encoded string from DataURL string
				base64Image = previewCanvas.canvas().toDataURL().split(",")[1];

				// clone canvas to become able to copy a smaller preview image
				var clonedCanvas = previewCanvas.clone();
				// set preview image dimensions to canvas element
				clonedCanvas
					.attr({
						width:	o.prevWidth,
						height:	o.prevHeight
					});

				// render canvas with preview demensions
				 clonedCanvas.canvas().drawImage(previewCanvas[0], 0, 0, o.prevWidth, o.prevHeight);
				// extract base64 encoded string with preview image information
				base64PreviewImage = clonedCanvas.canvas().toDataURL();

				previewCanvas.remove();

				// insert preview image to list image source
				var span = $('script[id=item_'+o.objectID+']').closest('div').find('span.epProductImage');
				span
					.children('img')
						.attr('src', base64PreviewImage)
						.css({
							width: span.width() + "px",
							height: span.height() + "px"
						});

				base64PreviewImage = base64PreviewImage.split(",")[1];

				// send base64 encoded strings via ajax
				ep.ajax({
					dataType: 'json',
					type: 'post',
					data: {
						'ViewAction'				: 'JSONViewResponse',
						'ObjectID'					: o.objectID,
						'ChangeAction'			: 'SaveCustomizedImage',
						'Base64Image'				: base64Image,
						'Base64PreviewImage': base64PreviewImage
					}
				})
				.done(function(data){
					// clean GUIDList
				})
				.fail(function(xhr,status,error){
					// ToDo: something more
					// $.debug.warn(error);
				});
			},

			// calculates dimensions of an image so that it fits to maxWidth/maxHeight values
			recalcDimensions = function(image, options){
				var $image = $(image),
						factor,
						dimension = {w: $image.width(), h: $image.height()};

				if(dimension.w > options.maxWidth){
					factor = options.maxWidth/dimension.w;
					dimension.w = options.maxWidth;
					dimension.h = factor * dimension.h;
				}

				if(dimension.h > options.maxHeight){
					factor = options.maxHeight/dimension.h;
					dimension.h = options.maxHeight;
					dimension.w = factor * dimension.w;
				}

				return dimension;
			};

		init();
	};
});