/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/product/inc/customizeSetting
 *
 * @revision        $Revision: 1.11 $
 */
/*global define*/
/*jslint browser: true, regexp:true, plusplus:true, vars: true*/
define("de_epages/product/inc/customizesettings", [
    "jquery",
    "ep",
    "util/base64",

    "jquery/tmpl",
    "jquery/ui/droppable",
    "jquery/ui/draggable",
    "jquery/ui/resizable",
    "de_epages/presentation/ui/uploader"
], function ($, ep, base64) {
    'use strict';

    return function (productID, selectors, attr, translation) {
        var areaField = $(selectors.areaInput),
            areaItems = $(selectors.areas),
            areaHolder = $(selectors.areaHolder),
            areaBox = $(selectors.areaBox),
            areaData = base64.decode(areaField.val().replace('\n', '') || "e30="),
            areaUpload = $(selectors.areaUploader),
            areaDrop,
            areaChange = function () {
                areaField.trigger({
                    type: 'change',
                    dojoconnect: true
                });
            },
            areaSave = function (elem, toggle) {
                var id = elem.data("areaId");

                if (toggle) {
                    areaData[id] = {
                        ProductID: productID,
                        AreaID: id,
                        FromTop: parseInt(elem.css("top"), 10),
                        FromLeft: parseInt(elem.css("left"), 10),
                        Width: parseInt(elem.width(), 10),
                        Height: parseInt(elem.height(), 10)
                    };
                } else {
                    delete areaData[id];
                }

                areaField.val(base64.encode(areaData));
            },
            areaRemove = function (elem) {
                elem.appendTo(areaHolder)
                    .css({
                        position: "relative",
                        top: "auto",
                        left: "auto",
                        width: "",
                        height: ""
                    })
                    .off("resize")
                    .empty()
                    .resizable("destroy");

                areaSave(elem, false);
                areaChange();
            },
            areaAdd = function (elem, data) {
                elem.appendTo(areaBox)
                    .css({
                        position: "absolute",
                        top: data.FromTop + "px",
                        left: data.FromLeft + "px",
                        width: data.Width + "px",
                        height: data.Height + "px"
                    })
                    .on("resize", function () {
                        areaSave(elem, true);
                        areaChange();
                        if (elem.closest("form")) {
                            elem.closest("form").trigger("change");
                        }
                    })
                    .resizable({
                        containment: "parent",
                        handles: "n, e, s, w, ne, se, sw, nw",
                        minHeight: 30,
                        minWidth: 40
                    });

                $('<a href="#">&#10006;</a>')
                    .on("click", function (event) {
                        event.preventDefault();
                        areaRemove(elem);
                    })
                    .appendTo(elem);

                areaSave(elem, true);
            },

            optionUID = 0,
            optionField = $(selectors.optionInput),
            optionData = base64.decode(optionField.val().replace('\n', '') || "e30="),
            optionList = $(selectors.optionList),
            optionItems,
            optionTextButton = $(selectors.optionAddText),
            optionFileButton = $(selectors.optionAddFile),
            optionHint = $(selectors.optionHint),
            optionMax,
            optionChange = function () {
                optionField.trigger({
                    type: 'change',
                    dojoconnect: true
                });
                if (optionField.closest("form")) {
                    optionField.closest("form").trigger("change");
                }
            },
            optionInit = function () {
                optionItems = optionList.children();
                optionMax = optionItems.length >= 5;
                optionTextButton.attr('disabled', optionMax);
                optionFileButton.attr('disabled', optionMax);
                if (optionMax) {
                    optionHint.show();
                }
            },
            optionRemove = function (event) {
                var elem = $(this),
                    item = elem.closest(selectors.optionList + " > *"),
                    id = item.data("optionId");

                item.remove();

                delete optionData[id];
                optionField.val(base64.encode(optionData));

                optionInit();
                optionChange();
            },
            optionSave = function (elem, id) {
                optionData[id] = optionData[id] || {
                    ProductID: productID,
                    OptionID: id
                };
                var data = optionData[id],
                    name = elem.attr('name').replace(/^Option\d+/, ''),
                    type = elem.attr('type'),
                    key = elem.data('optionKey'),
                    val;

                if (type === 'checkbox') {
                    val = elem.is(":checked") ? elem.val() : 0;
                } else if (type === 'radio') {
                    val = elem.is(":checked") ? elem.val() : elem.closest(selectors.optionList + " > *")
                        .find(":radio[name='" + elem.attr("name") + "']:checked")
                        .val();
                } else {
                    val = elem.val();
                }

                // fix number values
                val = $.isNumeric(val) ? parseFloat(val, 10) : val;

                // use sub keys
                if (key) {
                    data[name] = data[name] || {};
                    data[name][key] = val;
                } else {
                    data[name] = val;
                }

                optionField.val(base64.encode(optionData));
            },
            optionAdd = function (data, initialValidate) {
                if (optionMax) {
                    return;
                }

                data = typeof data === "string" ? {
                    Type: data,
                    ProductID: productID,
                    OptionID: (optionUID++)
                } : data;

                $(selectors.optionTemplate)
                    .tmpl([data])
                    .find('.ep-js')
                    .metaparse()
                    .end()
                    .find(selectors.optionRemove)
                    .on("click", optionRemove)
                    .end()
                    .appendTo(optionList)
                    .hide()
                    .find(':input')
                    .each(function () {
                        var elem = $(this);
                        if (initialValidate) {
                            elem.trigger("validate");
                            if (elem.closest("form")) {
                                ep(elem.closest("form")[0]).uiForm("setSaveButtonSaved");
                            }
                        }
                        optionSave(elem, data.OptionID);
                    })
                    .on("change keypress", function () {
                        optionSave($(this), data.OptionID);
                        optionChange();
                    })
                    .end()
                    .fadeIn(500);

                optionInit();
            },
            previewBox = $(selectors.previewBox),
            previewOptions = $(selectors.previewOptions),
            previewOff = $(selectors.previewOff),
            previewOn = $(selectors.previewOn),
            previewToggle = function () {
                var show = previewOn.prop('checked');

                previewBox[show ? 'fadeIn' : 'fadeOut']();
                previewOptions[show ? 'fadeIn' : 'fadeOut']();
            };

        // init interactive area handles
        areaItems.on("dragstop", function (event) {
            var originalEvent = event.originalEvent && (event.originalEvent.originalEvent || event.originalEvent),
                container = areaDrop ? $(areaDrop) : areaDrop,
                elem = $(this);

            if (container && container.is(selectors.areaHolder)) {
                areaRemove(elem);
            } else if (container && !elem.parent().is(selectors.areaBox)) {
                var offset = container.offset();

                areaAdd(elem, {
                    FromTop: (originalEvent.pageY - (originalEvent.offsetY || originalEvent.layerY) - offset.top),
                    FromLeft: (originalEvent.pageX - (originalEvent.offsetX || originalEvent.layerX) - offset.left),
                    Width: 40,
                    Height: 40
                });
            } else {
                areaSave(elem, true);
            }
            areaChange();

            if (elem.closest("form")) {
                elem.closest("form").trigger("change");
            }
        })
            .draggable({
                revert: "invalid",
                cursorAt: {
                    top: 21,
                    left: 21
                },
                start: function (event, ui) {
                    areaDrop = undefined;
                }
            });

        areaHolder.on("drop", function () {
            areaDrop = this;
        })
            .droppable({
                accept: selectors.areas
            });

        areaBox.on("drop", function () {
            areaDrop = this;
        })
            .droppable({
                tolerance: "fit",
                accept: selectors.areas
            });

        if (attr.AreaUploadData) {
            areaUpload.find('input:file')
                .presentationUiUploader({
                    accept: 'image/*',
                    buttonText: translation.buttonText,
                    data: attr.AreaUploadData,
                    callback: function (data) {
                        var file;

                        if (data && data[0] && data[0].success && data[0].success.uploadedFile) {
                            $('<img/>').attr({
                                src: data[0].success.uploadedFile,
                                alt: ''
                            }).prependTo(areaBox);

                            areaUpload.remove();
                        }
                    }
                });
        } else {
            areaUpload.remove();
        }

        // place to saved positions on start up
        areaItems.each(function () {
            var elem = $(this),
                data = areaData[elem.data("areaId")];

            if (data) {
                areaAdd(elem, data);
            }
        });

        previewOff.on("change", previewToggle);

        previewOn.on("change", previewToggle)
            .trigger("change");

        // init interactive option handles
        $(document)
            .on("metaparse", function () {
                optionTextButton.on("click", function () {
                    optionAdd("text");
                    optionChange();
                });

                optionFileButton.on("click", function () {
                    optionAdd("file");
                    optionChange();
                });

                // add saved option on start up
                $.each(ep.hashToArray(optionData, 'key')
                    .sort(), function (i, key) {
                        var data = optionData[key];

                        // setup the option unique ID
                        optionUID = optionUID <= data.OptionID ? data.OptionID + 1 : optionUID;
                        optionAdd(data, true);
                    });

                optionInit();
            });
    };
});